# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models
from django.utils import timezone

class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'File.created_datetime'
        db.add_column('database_files_file', 'created_datetime',
                      self.gf('django.db.models.fields.DateTimeField')(default=timezone.now, db_index=True),
                      keep_default=False)

        # Adding field 'File._content_hash'
        db.add_column('database_files_file', '_content_hash',
                      self.gf('django.db.models.fields.CharField')(max_length=128, null=True, db_column='content_hash', blank=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'File.created_datetime'
        db.delete_column('database_files_file', 'created_datetime')

        # Deleting field 'File._content_hash'
        db.delete_column('database_files_file', 'content_hash')


    models = {
        'database_files.file': {
            'Meta': {'object_name': 'File'},
            '_content': ('django.db.models.fields.TextField', [], {'db_column': "'content'"}),
            '_content_hash': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'db_column': "'content_hash'", 'blank': 'True'}),
            'created_datetime': ('django.db.models.fields.DateTimeField', [], {'default': 'timezone.now', 'db_index': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255', 'db_index': 'True'}),
            'size': ('django.db.models.fields.PositiveIntegerField', [], {})
        }
    }

    complete_apps = ['database_files']


from __future__ import print_function

import os
from optparse import make_option

from django.conf import settings
from django.core.files.storage import default_storage
from django.core.management.base import BaseCommand, CommandError
from django.db.models import FileField, ImageField, get_models

from database_files.models import File

class Command(BaseCommand):
    args = ''
    help = 'Deletes all files in the database that are not referenced by ' + \
        'any model fields.'
    option_list = BaseCommand.option_list + (
        make_option('--dryrun',
            action='store_true',
            dest='dryrun',
            default=False,
            help='If given, only displays the names of orphaned files ' + \
                'and does not delete them.'),
        )

    def handle(self, *args, **options):
        tmp_debug = settings.DEBUG
        settings.DEBUG = False
        names = set()
        dryrun = options['dryrun']
        try:
            for model in get_models():
                print('Checking model %s...' % (model,))
                for field in model._meta.fields:
                    if not isinstance(field, (FileField, ImageField)):
                        continue
                    # Ignore records with null or empty string values.
                    q = {'%s__isnull'%field.name:False}
                    xq = {field.name:''}
                    subq = model.objects.filter(**q).exclude(**xq)
                    subq_total = subq.count()
                    subq_i = 0
                    for row in subq.iterator():
                        subq_i += 1
                        if subq_i == 1 or not subq_i % 100:
                            print('%i of %i' % (subq_i, subq_total))
                        file = getattr(row, field.name)
                        if file is None:
                            continue
                        if not file.name:
                            continue
                        names.add(file.name)
            # Find all database files with names not in our list.
            print('Finding orphaned files...')
            orphan_files = File.objects.exclude(name__in=names).only('name', 'size')
            total_bytes = 0
            orphan_total = orphan_files.count()
            orphan_i = 0
            print('Deleting %i orphaned files...' % (orphan_total,))
            for f in orphan_files.iterator():
                orphan_i += 1
                if orphan_i == 1 or not orphan_i % 100:
                    print('%i of %i' % (orphan_i, orphan_total))
                total_bytes += f.size
                if dryrun:
                    print('File %s is orphaned.' % (f.name,))
                else:
                    print('Deleting orphan file %s...' % (f.name,))
                    default_storage.delete(f.name)
            print('%i total bytes in orphan files.' % total_bytes)
        finally:
            settings.DEBUG = tmp_debug



# encoding: utf-8
import datetime

from django.core.management import call_command
from django.db import models

from south.db import db
from south.v2 import SchemaMigration

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'File'
        db.create_table('database_files_file', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255, db_index=True)),
            ('size', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('_content', self.gf('django.db.models.fields.TextField')(db_column='content')),
        ))
        db.send_create_signal('database_files', ['File'])

    def backwards(self, orm):
        
        # Deleting model 'File'
        db.delete_table('database_files_file')

    models = {
        'database_files.file': {
            'Meta': {'object_name': 'File'},
            '_content': ('django.db.models.fields.TextField', [], {'db_column': "'content'"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255', 'db_index': 'True'}),
            'size': ('django.db.models.fields.PositiveIntegerField', [], {})
        }
    }

    complete_apps = ['database_files']



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding index on 'File', fields ['_content_hash']
        db.create_index('database_files_file', ['content_hash'])

        # Adding index on 'File', fields ['size']
        db.create_index('database_files_file', ['size'])


    def backwards(self, orm):
        # Removing index on 'File', fields ['size']
        db.delete_index('database_files_file', ['size'])

        # Removing index on 'File', fields ['_content_hash']
        db.delete_index('database_files_file', ['content_hash'])


    models = {
        'database_files.file': {
            'Meta': {'object_name': 'File'},
            '_content': ('django.db.models.fields.TextField', [], {'db_column': "'content'"}),
            '_content_hash': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '128', 'null': 'True', 'db_column': "'content_hash'", 'blank': 'True'}),
            'created_datetime': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'db_index': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255', 'db_index': 'True'}),
            'size': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'})
        }
    }

    complete_apps = ['database_files']


from __future__ import print_function
import os

import six
from six import StringIO

from django.conf import settings
from django.core import files
from django.core.files.storage import FileSystemStorage
from django.core.urlresolvers import reverse

from database_files import models
from database_files import utils

class DatabaseStorage(FileSystemStorage):
    
    def _generate_name(self, name, pk):
        """
        Replaces the filename with the specified pk and removes any dir
        """
        #dir_name, file_name = os.path.split(name)
        #file_root, file_ext = os.path.splitext(file_name)
        #return '%s%s' % (pk, file_name)
        return name
    
    def _open(self, name, mode='rb'):
        """
        Open file with filename `name` from the database.
        """
        try:
            # Load file from database.
            f = models.File.objects.get_from_name(name)
            content = f.content
            size = f.size
            if settings.DB_FILES_AUTO_EXPORT_DB_TO_FS \
            and not utils.is_fresh(f.name, f.content_hash):
                # Automatically write the file to the filesystem
                # if it's missing and exists in the database.
                # This happens if we're using multiple web servers connected
                # to a common databaes behind a load balancer.
                # One user might upload a file from one web server, and then
                # another might access if from another server.
                utils.write_file(f.name, f.content)
        except models.File.DoesNotExist:
            # If not yet in the database, check the local file system
            # and load it into the database if present.
            fqfn = self.path(name)
            if os.path.isfile(fqfn):
                #print('Loading file into database.')
                self._save(name, open(fqfn, mode))
                fh = super(DatabaseStorage, self)._open(name, mode)
                content = fh.read()
                size = fh.size
            else:
                # Otherwise we don't know where the file is.
                return
        # Normalize the content to a new file object.
        #fh = StringIO(content)
        fh = six.BytesIO(content)
        fh.name = name
        fh.mode = mode
        fh.size = size
        o = files.File(fh)
        return o
    
    def _save(self, name, content):
        """
        Save file with filename `name` and given content to the database.
        """
        full_path = self.path(name)
        try:
            size = content.size
        except AttributeError:
            size = os.path.getsize(full_path)
        content.seek(0)
        content = content.read()
        f = models.File.objects.create(
            content=content,
            size=size,
            name=name,
        )
        # Automatically write the change to the local file system.
        if settings.DB_FILES_AUTO_EXPORT_DB_TO_FS:
            utils.write_file(name, content, overwrite=True)
        #TODO:add callback to handle custom save behavior?
        return self._generate_name(name, f.pk)
    
    def exists(self, name):
        """
        Returns true if a file with the given filename exists in the database.
        Returns false otherwise.
        """
        if models.File.objects.filter(name=name).count() > 0:
            return True
        return super(DatabaseStorage, self).exists(name)
    
    def delete(self, name):
        """
        Deletes the file with filename `name` from the database and filesystem.
        """
        try:
            models.File.objects.get_from_name(name).delete()
            hash_fn = utils.get_hash_fn(name)
            if os.path.isfile(hash_fn):
                os.remove(hash_fn)
        except models.File.DoesNotExist:
            pass
        return super(DatabaseStorage, self).delete(name)
    
    def url(self, name):
        """
        Returns the web-accessible URL for the file with filename `name`.
        """
        return settings.DATABASE_FILES_URL_METHOD(name)
    
    def size(self, name):
        """
        Returns the size of the file with filename `name` in bytes.
        """
        full_path = self.path(name)
        try:
            return models.File.objects.get_from_name(name).size
        except models.File.DoesNotExist:
            return super(DatabaseStorage, self).size(name)



from __future__ import print_function

import os

from django.conf import settings
from django.core.files.storage import default_storage
from django.core.management.base import BaseCommand, CommandError
from django.db.models import FileField, ImageField, get_models

from optparse import make_option

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('-m', '--models',
            dest='models', default='',
            help='A list of models to search for file fields. Default is all.'),
    )
    help = 'Loads all files on the filesystem referenced by FileFields ' + \
        'or ImageFields into the database. This should only need to be ' + \
        'done once, when initially migrating a legacy system.'

    def handle(self, *args, **options):
        show_files = int(options.get('verbosity', 1)) >= 2
        all_models = [
            _.lower().strip()
            for _ in options.get('models', '').split()
            if _.strip()
        ]
        tmp_debug = settings.DEBUG
        settings.DEBUG = False
        try:
            broken = 0 # Number of db records referencing missing files.
            for model in get_models():
                key = "%s.%s" % (model._meta.app_label,model._meta.module_name)
                key = key.lower()
                if all_models and key not in all_models:
                    continue
                for field in model._meta.fields:
                    if not isinstance(field, (FileField, ImageField)):
                        continue
                    if show_files:
                        print(model.__name__, field.name)
                    # Ignore records with null or empty string values.
                    q = {'%s__isnull'%field.name:False}
                    xq = {field.name:''}
                    for row in model.objects.filter(**q).exclude(**xq):
                        try:
                            file = getattr(row, field.name)
                            if file is None:
                                continue
                            if not file.name:
                                continue
                            if show_files:
                                print("\t",file.name)
                            if file.path and not os.path.isfile(file.path):
                                if show_files:
                                    print("Broken:",file.name)
                                broken += 1
                                continue
                            file.read()
                            row.save()
                        except IOError:
                            broken += 1
            if show_files:
                print('-'*80)
                print('%i broken' % (broken,))
        finally:
            settings.DEBUG = tmp_debug




try:
    # Removed in Django 1.6
    from django.conf.urls.defaults import patterns, url
except ImportError:
    from django.conf.urls import patterns, url

urlpatterns = patterns('',
#    url(r'^files/(?P<name>.+)$',
#        'database_files.views.serve',
#        name='database_file'),
    url(r'^files/(?P<name>.+)$',
        'database_files.views.serve_mixed',
        name='database_file'),
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages, Command

import database_files

try:
    from pypandoc import convert
    read_md = lambda f: convert(f, 'rst')
except:
    print("Warning: pypandoc module not found, could not convert "
        "Markdown to RST")
    read_md = lambda f: open(f, 'r').read()

def get_reqs(test=False, pv=None):
    reqs = [
        'Django>=1.4',
        'six>=1.7.2',
    ]
    if test:
        #TODO:remove once Django 1.7 south integration becomes main-stream?
        if not pv:
            reqs.append('South>=1.0')
        elif pv <= 3.2:
            # Note, South dropped Python3 support after 0.8.4...
            reqs.append('South==0.8.4')
    return reqs

class TestCommand(Command):
    description = "Runs unittests."
    user_options = [
        ('name=', None,
         'Name of the specific test to run.'),
        ('virtual-env-dir=', None,
         'The location of the virtual environment to use.'),
        ('pv=', None,
         'The version of Python to use. e.g. 2.7 or 3'),
    ]
    
    def initialize_options(self):
        self.name = None
        self.virtual_env_dir = '.env%s'
        self.pv = 0
        self.versions = [
            2.7,
            3,
            #3.3,#TODO?
        ]
        
    def finalize_options(self):
        pass
    
    def build_virtualenv(self, pv):
        virtual_env_dir = self.virtual_env_dir % pv
        kwargs = dict(virtual_env_dir=virtual_env_dir, pv=pv)
        if not os.path.isdir(virtual_env_dir):
            cmd = 'virtualenv -p /usr/bin/python{pv} {virtual_env_dir}'.format(**kwargs)
            print(cmd)
            os.system(cmd)
            
            cmd = '{virtual_env_dir}/bin/easy_install -U distribute'.format(**kwargs)
            print(cmd)
            os.system(cmd)
            
            for package in get_reqs(test=True, pv=float(pv)):
                kwargs['package'] = package
                cmd = '{virtual_env_dir}/bin/pip install -U {package}'.format(**kwargs)
                print(cmd)
                os.system(cmd)
    
    def run(self):
        versions = self.versions
        if self.pv:
            versions = [self.pv]
        
        for pv in versions:
            
            self.build_virtualenv(pv)
            kwargs = dict(
                pv=pv,
                virtual_env_dir=self.virtual_env_dir % pv,
                name=self.name)
                
            if self.name:
                cmd = '{virtual_env_dir}/bin/django-admin.py test --pythonpath=. --settings=database_files.tests.settings database_files.tests.tests.{name}'.format(**kwargs)
            else:
                cmd = '{virtual_env_dir}/bin/django-admin.py test --pythonpath=. --settings=database_files.tests.settings database_files.tests'.format(**kwargs)

            print(cmd)
            ret = os.system(cmd)
            if ret:
                return

try:
    long_description = read_md('README.md')
except IOError:
    long_description = ''

setup(
    name='django-database-files-3000',
    version=database_files.__version__,
    description='A storage system for Django that stores uploaded files in both the database and file system.',
    long_description=long_description,
    author='Chris Spencer',
    author_email='chrisspen@gmail.com',
    url='http://github.com/chrisspen/django-database-files-3000',
    packages=[
        'database_files',
        'database_files.management',
        'database_files.management.commands',
        'database_files.migrations',
    ],
    #https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.0',
        'Programming Language :: Python :: 3.2',
    ],
    install_requires = get_reqs(),
    cmdclass={
        'test': TestCommand,
    },
)



from __future__ import print_function

from optparse import make_option

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError

from database_files.models import File

class Command(BaseCommand):
    args = '<filename 1> <filename 2> ... <filename N>'
    help = 'Regenerates hashes for files. If no filenames given, ' + \
        'rehashes everything.'
    option_list = BaseCommand.option_list + (
#        make_option('--dryrun',
#            action='store_true',
#            dest='dryrun',
#            default=False,
#            help='If given, only displays the names of orphaned files ' + \
#                'and does not delete them.'),
        )

    def handle(self, *args, **options):
        tmp_debug = settings.DEBUG
        settings.DEBUG = False
        try:
            q = File.objects.all()
            if args:
                q = q.filter(name__in=args)
            total = q.count()
            i = 1
            for f in q.iterator():
                print('%i of %i: %s' % (i, total, f.name))
                f._content_hash = None
                f.save()
                i += 1
        finally:
            settings.DEBUG = tmp_debug
